<?php

namespace QUI\Verification;

use QUI;
use QUI\Verification\Entity\AbstractVerification;
use QUI\Verification\Entity\PhoneNumberVerification;
use QUI\Verification\Enum\VerificationErrorReason;
use QUI\Verification\Enum\VerificationStatus;
use QUI\Verification\Exception\InvalidVerificationCodeException;
use QUI\Verification\Exception\VerificationAlreadyVerifiedException;
use QUI\Verification\Exception\VerificationExpiredException;
use QUI\Verification\Interface\PhoneNumberVerificationHandlerInterface;
use QUI\Verification\Interface\VerificationRepositoryInterface;
use QUI\Verification\Interface\VerifierInterface;

class Verifier implements VerifierInterface
{
    /**
     * @param VerificationRepositoryInterface|null $verificationRepository
     */
    public function __construct(protected ?VerificationRepositoryInterface $verificationRepository = null)
    {
        if (is_null($this->verificationRepository)) {
            $this->verificationRepository = new VerificationRepository();
        }
    }

    /**
     * @param AbstractVerification $verification
     * @param string $verificationCode
     * @return void
     *
     * @throws InvalidVerificationCodeException
     * @throws VerificationAlreadyVerifiedException
     * @throws VerificationExpiredException
     */
    public function verifyVerificationCode(AbstractVerification $verification, string $verificationCode): void
    {
        if ($verification->status === VerificationStatus::VERIFIED) {
            throw new VerificationAlreadyVerifiedException(
                "Verification {$verification->uuid} is already verified."
            );
        }

        if ($verification->status === VerificationStatus::EXPIRED) {
            throw new VerificationExpiredException(
                "Verification {$verification->uuid} is expired."
            );
        }

        // If expired but verification instance does not reflect that -> update state
        if ($verification->isValid() === false) {
            $verification->status = VerificationStatus::EXPIRED;
            $this->verificationRepository->update($verification);

            throw new VerificationExpiredException(
                "Verification {$verification->uuid} is expired."
            );
        }

        if ($verification->verificationCode !== $verificationCode) {
            throw new InvalidVerificationCodeException(
                "Verification code {$verificationCode} is invalid for verification {$verification->uuid}."
            );
        }

        $verification->status = VerificationStatus::VERIFIED;
        $this->verificationRepository->update($verification);
    }

    /**
     * @param PhoneNumberVerification $verification
     * @param string $verificationCode
     * @return void
     *
     * @throws \Doctrine\DBAL\Exception
     */
    public function verifyPhoneNumberVerification(PhoneNumberVerification $verification, string $verificationCode): void
    {
        $errorReason = null;

        try {
            $this->verifyVerificationCode($verification, $verificationCode);
        } catch (VerificationAlreadyVerifiedException $exception) {
            QUI\System\Log::writeDebugException($exception);
            $errorReason = VerificationErrorReason::ALREADY_VERIFIED;
        } catch (VerificationExpiredException $exception) {
            QUI\System\Log::writeDebugException($exception);
            $errorReason = VerificationErrorReason::EXPIRED;
        } catch (InvalidVerificationCodeException $exception) {
            QUI\System\Log::writeDebugException($exception);
            $errorReason = VerificationErrorReason::INVALID_CODE;
        }

        /** @var PhoneNumberVerificationHandlerInterface $verificationHandler */
        $verificationHandler = $this->verificationRepository->getVerificationHandler($verification);

        if (is_null($errorReason)) {
            $verificationHandler->onSuccess($verification);
        } else {
            $verificationHandler->onError($verification, $errorReason);
        }
    }
}
